@@ -11,7 +11,7 @@ use strict;
 use vars qw($VERSION);
 use Carp ();
 
-$VERSION = '1.21';
+$VERSION = '1.23';
 
 sub PV  { 0 }
 sub IV  { 1 }
@@ -82,6 +82,8 @@ my %def_attr = (
     types               => undef,
     verbatim            => 0,
     blank_is_undef      => 0,
+    empty_is_undef      => 0,
+    auto_diag           => 0,
 
     _EOF                => 0,
     _STATUS             => undef,
@@ -137,9 +139,12 @@ sub version {
 ################################################################################
 sub new {
     my $proto = shift;
-    my $attr  = shift || {};
+    my $attr  = @_ > 0 ? shift : {};
 
-    $last_new_error = 'usage: my $csv = Text::CSV_PP->new ([{ option => value, ... }]);';
+    $last_new_error   = 'usage: my $csv = Text::CSV_PP->new ([{ option => value, ... }]);';
+    $last_new_err_num = 1000;
+
+    return unless ( defined $attr and ref($attr) eq 'HASH' );
 
     my $class = ref($proto) || $proto or return;
     my $self  = { %def_attr };
@@ -203,8 +208,11 @@ sub error_diag {
 
     my $diagobj = bless \@diag, 'Text::CSV::ErrorDiag';
 
-    unless (defined $context) {	# Void context
-        print STDERR "# CSV_PP ERROR: ", $diag[0], " - $diag[1]\n" if ( $diag[0] );
+    unless (defined $context) { # Void context
+        if ( $diag[0] ) {
+            my $msg = "# CSV_PP ERROR: " . $diag[0] . " - $diag[1]\n";
+            $self->{auto_diag} > 1 ? die $msg : warn $msg;
+        }
         return;
     }
 
@@ -239,8 +247,8 @@ sub _combine {
     $self->{_STRING}      = '';
     $self->{_STATUS}      = 0;
 
-    my ($always_quote, $binary, $quot, $sep, $esc)
-            = @{$self}{qw/always_quote binary quote_char sep_char escape_char/};
+    my ($always_quote, $binary, $quot, $sep, $esc, $empty_is_undef)
+            = @{$self}{qw/always_quote binary quote_char sep_char escape_char empty_is_undef/};
 
     if(!defined $quot){ $quot = ''; }
 
@@ -281,12 +289,15 @@ sub _combine {
             $must_be_quoted++ if ( $column =~ s/\0/${esc}0/g || $column =~ /[\x00-\x1f\x7f-\xa0]/ );
         }
 
+        if ( $empty_is_undef and defined $column and not length $column ) {
+        }
+        #elsif($always_quote or $must_be_quoted){
         if($always_quote or $must_be_quoted){
             $column = $quot . $column . $quot;
         }
+
     }
 
-#    $self->{_STRING} = \do { join($sep, @part) . $self->{eol} };
     $self->{_STRING} = \do { join($sep, @part) . ( defined $self->{eol} ? $self->{eol} : '' ) };
     $self->{_STATUS} = 1;
 
@@ -300,20 +311,17 @@ my %allow_eol = ("\r" => 1, "\r\n" => 1, "\n" => 1, "" => 1);
 *parse = \&_parse;
 
 sub _parse {
-    my ($self, $line, $useio) = @_;
-    my $data;
-
-    if ( $useio ) {
-        return if eof($useio);
-        $line = $useio->getline();
-    }
+    my ($self, $line) = @_;
 
     @{$self}{qw/_STRING _FIELDS _STATUS _ERROR_INPUT/} = ( \do{ defined $line ? "$line" : undef }, undef, 0, $line );
 
     return 0 if(!defined $line);
 
-    my ($binary, $quot, $sep, $esc, $types, $keep_meta_info, $allow_whitespace, $eol, $blank_is_undef)
-         = @{$self}{qw/binary quote_char sep_char escape_char types keep_meta_info allow_whitespace eol blank_is_undef/};
+    my ($binary, $quot, $sep, $esc, $types, $keep_meta_info, $allow_whitespace, $eol, $blank_is_undef, $empty_is_undef)
+         = @{$self}{
+            qw/binary quote_char sep_char escape_char types keep_meta_info allow_whitespace eol blank_is_undef empty_is_undef/
+           };
+
     $sep  = "\0" unless (defined $sep);
     $esc  = "\0" unless (defined $esc);
     $quot = ''   unless (defined $quot);
@@ -363,10 +371,6 @@ sub _parse {
 
     for my $col ( $line =~ /$re_split/g ) {
 
-        if ( $useio ) {
-#            print "=$line=";
-        }
-
         if ($keep_meta_info) {
             $flag = 0x0000;
             $flag |= IS_BINARY if ($col =~ /[^\x09\x20-\x7E]/);
@@ -435,9 +439,14 @@ sub _parse {
 
             $col =~ s{$re_esc}{$1 eq '0' ? "\0" : $1}eg;
 
+            if ( $empty_is_undef and length($col) == 0 ) {
+                $col = undef;
+            }
+
             if ($types and $types->[$i]) { # IV or NV
                 _check_type(\$col, $types->[$i]);
             }
+
         }
 
         # quoted but invalid
@@ -445,13 +454,6 @@ sub _parse {
         elsif ($col =~ $re_invalid_quot) {
 
             unless ($self->{allow_loose_quotes} and $col =~ /$re_quot_char/) {
-
-                if ( $useio ) {
-                    return if eof($useio);
-                    $line .= $useio->getline();
-                    next;
-                }
-
                 $self->_set_error_diag(
                       $col =~ /^\Q$quot\E(.*)\Q$quot\E.$/s  ? (2011, $pos - 2)
                     : $col =~ /^$re_quot_char/              ? (2027, $pos - 1)
@@ -472,9 +474,7 @@ sub _parse {
         else {
 
             if (!$self->{verbatim} and $col =~ /\r\n|\n/) {
-                unless (defined $eol and !$allow_eol{$eol}) {
-                    $col =~ s/(?:\r\n|\n).*//sm;
-                }
+                $col =~ s/(?:\r\n|\n).*$//sm;
             }
 
             if ($col =~ /\Q$esc\E\r$/) { # for t/15_flags : test 165 'ESC CR' at line 203
@@ -493,6 +493,10 @@ sub _parse {
                 $col = undef;
             }
 
+            if ( $empty_is_undef and length($col) == 0 ) {
+                $col = undef;
+            }
+
         }
 
         push @part,$col;
@@ -588,117 +592,79 @@ sub getline {
 
     $self->{_EOF} = eof($io) ? 1 : '';
 
-#    $self->_parse( undef, $io ) or return;
-
-
-    my $line = $io->getline();
     my $quot = $self->{quote_char};
     my $sep  = $self->{sep_char};
-    my $eol  = $self->{eol};
-
     my $re   = qr/(?:\Q$quot\E)/;
 
-    if ( defined $line and $line =~ /${re}0/ ) {
-#        print "=$line=\n";
+    local $/ = "\r" if $self->{_AUTO_DETECT_CR};
+
+    my $line = $io->getline();
+    my $eol  = $self->{eol};
+
+    # AUTO DETECTION EOL CR
+    if ( defined $line and defined $eol and $eol eq '' and $line =~ /[^\r]\r[^\r\n]/ and eof ) {
+        $self->{_AUTO_DETECT_CR} = 1;
+        $self->{eol} = "\r";
+        seek( $io, 0, 0 ); # restart
+        return $self->getline( $io );
+    }
+
+    if ( defined $line and $line =~ /${re}0/ ) { # null containing line
+        my $auto_diag = $self->auto_diag;
+        $self->auto_diag( 0 ) if ( $auto_diag ); # stop auto error diag
+
         while ( not $self->_parse($line) and !eof($io) ) {
-#            print $self->error_diag, "???\n";
-#            print "\t=$line=\n";
             $line .= $io->getline();
         }
-    }
-    else {
-        $line .= $io->getline() while ( defined $line and scalar(my @list = $line =~ /$re/g) % 2 and !eof($io) );
 
-        if (defined $eol and defined $line) {
-            $line =~ s/\Q$eol\E$//;
-        }
+        $self->auto_diag( $auto_diag ) if ( $auto_diag ); # restore
 
-        $self->_parse($line) or return;
+        return $self->_return_getline_result;
     }
 
+    $line .= $io->getline() while ( defined $line and scalar(my @list = $line =~ /$re/g) % 2 and !eof($io) );
 
-
-=pod
-
-    if ( $self->binary ) {
-        my $start = 0;
-        my $re   = qr/(?:\Q$quot\E)/;
-
-        while (
-                ( defined $line and !eof($io) ) and (
-                    ( $line !~ /[^$re]${re}0/ and scalar(my @list = $line =~ /$re/g) % 2 )
-                        or 
-                    ( scalar(my @list = $line =~ /(?![^$re]${re}0)/g) % 2 )
-#                        or 
-#                    ( scalar(my @list = $line =~ /(?:^|\Q$sep\E) *(?:\Q$quot\E)/g) % 2 )
-                )
-        )
-        {
-            $line .= $io->getline()
-        }
-#        if ( defined $line and $line =~ /(?:^|\Q$sep\E) *(?:\Q$quot\E)/ and !eof($io)  ){
-#            print "firstline\n";
-#        } # first line
-    }
-    else {
-        my $re   = qr/(?:\Q$quot\E)/;
-        $line .= $io->getline() while ( defined $line and scalar(my @list = $line =~ /$re/g) % 2 and !eof($io) );
-    }
-
-    if (defined $eol and defined $line) {
+    if ( $self->{verbatim} and defined $eol and defined $line ) { # VERBATIM MODE
         $line =~ s/\Q$eol\E$//;
     }
-print "=$line=\n";
-    $self->_parse($line) or return;
 
-#=pod
-
-#    my $re   = qr/(?:\Q$quot\E)/;
-#    my $re   = $self->binary ? qr/(?:\Q$quot\E)(?!0)/ : qr/(?:\Q$quot\E)/;
-#    my $re   = $self->binary ? qr/(?:^|\Q$sep\E) *(?:\Q$quot\E)|(?:\Q$quot\E)(?:(?![0\Q$quot\E])|\Q$sep\E|$)/ : qr/(?:\Q$quot\E)/;
-#    my $re   = $self->binary ? qr/(?:\Q$quot\E)|(?:\Q$quot\E)(?![0\Q$quot\E])/ : qr/(?:\Q$quot\E)/;
-    my $re   = $self->binary ? qr/(?:^|\Q$sep\E) *(?:\Q$quot\E)|(?:\Q$quot$quot\E)*(?:\Q$quot\E)0/ : qr/(?:\Q$quot\E)/;
-
-#    $line .= $io->getline() while ( defined $line and scalar(my @list = $line =~ /$re/g) % 2 and !eof($io) );
-#print "=$line=\n";
-#    my $eol = $self->{eol};
-
-#    $line = $io->getline();
-#     while ( defined $line and scalar(my @list = $line =~ /$re/g) % 2 and !eof($io) );
-
-#    if (defined $eol and defined $line) {
-#        $line =~ s/\Q$eol\E$//;
-#    }
-#print "=====$line======\n";
-#    $self->_parse($line,1) or return;
-
-=cut
-
-    if ( $self->{_BOUND_COLUMNS} ) {
-        my @vals  = $self->_fields();
-        my ( $max, $count ) = ( scalar @vals, 0 );
-
-        if ( @{ $self->{_BOUND_COLUMNS} } < $max ) {
-                $self->_set_error_diag(3006);
-                return;
-        }
-
-        for ( my $i = 0; $i < $max; $i++ ) {
-            my $bind = $self->{_BOUND_COLUMNS}->[ $i ];
-            if ( Scalar::Util::readonly( $$bind ) ) {
-                $self->_set_error_diag(3008);
-                return;
-            }
-            $$bind = $vals[ $i ];
-        }
-
-        return [];
-    }
-    else {
-        [ $self->_fields() ];
-    }
+    $self->_parse($line);
 
+    return $self->_return_getline_result();
 }
+
+
+sub _return_getline_result {
+
+    if ( eof ) {
+        $_[0]->{_AUTO_DETECT_CR} = 0;
+    }
+
+    return unless $_[0]->{_STATUS};
+
+    return [ $_[0]->_fields() ] unless $_[0]->{_BOUND_COLUMNS};
+
+    my @vals  = $_[0]->_fields();
+    my ( $max, $count ) = ( scalar @vals, 0 );
+
+    if ( @{ $_[0]->{_BOUND_COLUMNS} } < $max ) {
+            $_[0]->_set_error_diag(3006);
+            return;
+    }
+
+    for ( my $i = 0; $i < $max; $i++ ) {
+        my $bind = $_[0]->{_BOUND_COLUMNS}->[ $i ];
+        if ( Scalar::Util::readonly( $$bind ) ) {
+            $_[0]->_set_error_diag(3008);
+            return;
+        }
+        $$bind = $vals[ $i ];
+    }
+
+    return [];
+}
+
+
 ################################################################################
 # getline_hr
 ################################################################################
@@ -834,13 +800,15 @@ sub _set_error_diag {
         $_[0]->{_ERROR_POS} = $pos;
     }
 
+    $self->error_diag() if ( $error and $self->{auto_diag} );
+
     return;
 }
 ################################################################################
 
 BEGIN {
-    for my $method (qw/sep_char always_quote binary
-                        keep_meta_info allow_loose_quotes allow_loose_escapes verbatim blank_is_undef/) {
+    for my $method ( qw/sep_char always_quote binary keep_meta_info allow_loose_quotes allow_loose_escapes
+                            verbatim blank_is_undef empty_is_undef auto_diag/ ) {
         eval qq|
             sub $method {
                 \$_[0]->{$method} = defined \$_[1] ? \$_[1] : 0 if (\@_ > 1);
@@ -973,6 +941,12 @@ provides facilities for the composition and decomposition of
 comma-separated values. As its name suggests, L<Text::CSV_XS>
 is a XS module and Text::CSV_PP is a Puer Perl one.
 
+=head1 VERSION
+
+    1.22
+
+This module is compatible with Text::CSV_XS B<0.68> and later.
+
 =head2 Unicode (UTF8)
 
 On parsing (both for C<getline ()> and C<parse ()>), if the source is
@@ -991,7 +965,12 @@ See to L<Text::CSV_XS>.
 
 =head2 version ()
 
-(Class method) Returns the current module version.
+(Class method) Returns the current backend module version.
+If you want the module version, you can use the C<VERSION> method,
+
+ print Text::CSV->VERSION;      # This module version
+ print Text::CSV->version;      # The version of the worker module
+                                # same as Text::CSV->backend->version
 
 =head2 new (\%attr)
 
@@ -1038,7 +1017,7 @@ So lines like:
 are now correctly parsed, even though it violates the CSV specs.
 
 Note that B<all> whitespace is stripped from start and end of each
-field. That would make is more a I<feature> than a way to be able
+field. That would make it more a I<feature> than a way to be able
 to parse bad CSV lines, as
 
  1,   2.0,  3,   ape  , monkey
@@ -1069,6 +1048,20 @@ be parsed as
 
  ("1", "", undef, " ", "2")
 
+=item empty_is_undef
+
+Going one step further than C<blank_is_undef>, this attribute converts
+all empty fields to undef, so
+
+ 1,"",," ",2
+
+is read as
+
+ (1, undef, undef, " ", 2)
+
+Note that this only effects fields that are I<realy> empty, not fields
+that are empty after stripping allowed whitespace. YMMV.
+
 =item quote_char
 
 The char used for quoting fields containing blanks, by default the
@@ -1126,7 +1119,7 @@ The escape character can not be equal to the separation character.
 By default, parsing fields that have C<escape_char> characters that
 escape characters that do not need to be escaped, like:
 
- my $csv = Text::CSV_PP->new ({ escape_char => "\\" });
+ my $csv = Text::CSV->new ({ escape_char => "\\" });
  $csv->parse (qq{1,"my bar\'s",baz,42});
 
 would result in a parse error. Though it is still bad practice to
@@ -1139,6 +1132,11 @@ If this attribute is TRUE, you may use binary characters in quoted fields,
 including line feeds, carriage returns and NULL bytes. (The latter must
 be escaped as C<"0>.) By default this feature is off.
 
+If a string is marked UTF8, binary will be turned on automatically when
+binary characters other than CR or NL are encountered. Note that a simple
+string like C<"\x{00a0}"> might still be binary, but not marked UTF8, so
+setting C<{ binary => 1 }> is still a wise option.
+
 =item types
 
 A set of column types; this attribute is immediately passed to the
@@ -1173,7 +1171,7 @@ special when this flag is set, and be dealt with as being ordinary
 binary characters. This will ease working with data with embedded
 newlines.
 
-When C<verbatim> is used with C<getline ()>, getline
+When C<verbatim> is used with C<getline ()>, C<getline ()>
 auto-chomp's every line.
 
 Imagine a file format like
@@ -1185,7 +1183,7 @@ is a ^ (caret). None of the fields is quoted, but embedded binary
 data is likely to be present. With the specific line ending, that
 shouldn't be too hard to detect.
 
-By default, Text::CSV_PP' parse function however is instructed to only
+By default, Text::CSV' parse function however is instructed to only
 know about "\n" and "\r" to be legal line endings, and so has to deal
 with the embedded newline as a real end-of-line, so it can scan the next
 line if binary is true, and the newline is inside a quoted field.
@@ -1195,6 +1193,17 @@ as if \n is just nothing more than a binary character.
 For parse () this means that the parser has no idea about line ending
 anymore, and getline () chomps line endings on reading.
 
+=item auto_diag
+
+Set to true will cause C<error_diag ()> to be automatically be called
+in void context upon errors.
+
+If set to a value greater than 1, it will die on errors instead of
+warn.
+
+To check future plans and a difference in XS version,
+please see to L<Text::CSV_XS/auto_diag>.
+
 =back
 
 To sum it up,
@@ -1215,9 +1224,12 @@ is equivalent to
      allow_loose_escapes => 0,
      allow_whitespace    => 0,
      blank_is_undef      => 0,
+     empty_is_undef      => 0,
      verbatim            => 0,
+     auto_diag           => 0,
      });
 
+
 For all of the above mentioned flags, there is an accessor method
 available where you can inquire for the current value, or change
 the value
@@ -1232,13 +1244,37 @@ the available CSV object, there is no harm in changing them.
 If the C<new ()> constructor call fails, it returns C<undef>, and makes
 the fail reason available through the C<error_diag ()> method.
 
- $csv = Text::CSV_PP->new ({ ecs_char => 1 }) or
-     die Text::CSV_PP->error_diag ();
+ $csv = Text::CSV->new ({ ecs_char => 1 }) or
+     die "" . Text::CSV->error_diag ();
 
 C<error_diag ()> will return a string like
 
  "INI - Unknown attribute 'ecs_char'"
 
+=head2 print
+
+ $status = $csv->print ($io, $colref);
+
+Similar to C<combine () + string () + print>, but more efficient. It
+expects an array ref as input (not an array!) and the resulting string is
+not really created (XS version), but immediately written to the I<$io> object, typically
+an IO handle or any other object that offers a I<print> method. Note, this
+implies that the following is wrong in perl 5.005_xx and older:
+
+ open FILE, ">", "whatever";
+ $status = $csv->print (\*FILE, $colref);
+
+as in perl 5.005 and older, the glob C<\*FILE> is not an object, thus it
+doesn't have a print method. The solution is to use an IO::File object or
+to hide the glob behind an IO::Wrap object. See L<IO::File> and L<IO::Wrap>
+for details.
+
+For performance reasons the print method doesn't create a result string.
+(If its backend is PP version, result strings are created internally.)
+In particular the I<$csv-E<gt>string ()>, I<$csv-E<gt>status ()>,
+I<$csv->fields ()> and I<$csv-E<gt>error_input ()> methods are meaningless
+after executing this method.
+
 =head2 combine
 
  $status = $csv->combine (@columns);
@@ -1250,28 +1286,6 @@ retrieve the resultant CSV string.  Upon failure, the value returned by
 C<string ()> is undefined and C<error_input ()> can be called to retrieve an
 invalid argument.
 
-=head2 print
-
- $status = $csv->print ($io, $colref);
-
-Similar to combine, but it expects an array ref as input (not an array!)
-and the resulting string is not really created, but immediately written
-to the I<$io> object, typically an IO handle or any other object that
-offers a I<print> method. Note, this implies that the following is wrong:
-
- open FILE, ">", "whatever";
- $status = $csv->print (\*FILE, $colref);
-
-The glob C<\*FILE> is not an object, thus it doesn't have a print
-method. The solution is to use an IO::File object or to hide the
-glob behind an IO::Wrap object. See L<IO::File(3)> and L<IO::Wrap(3)>
-for details.
-
-For performance reasons the print method doesn't create a result string.
-In particular the I<$csv-E<gt>string ()>, I<$csv-E<gt>status ()>,
-I<$csv->fields ()> and I<$csv-E<gt>error_input ()> methods are meaningless
-after executing this method.
-
 =head2 string
 
  $line = $csv->string ();
@@ -1279,20 +1293,6 @@ after executing this method.
 This object function returns the input to C<parse ()> or the resultant CSV
 string of C<combine ()>, whichever was called more recently.
 
-=head2 parse
-
- $status = $csv->parse ($line);
-
-This object function decomposes a CSV string into fields, returning
-success or failure.  Failure can result from a lack of argument or the
-given CSV string is improperly formatted.  Upon success, C<fields ()> can
-be called to retrieve the decomposed fields .  Upon failure, the value
-returned by C<fields ()> is undefined and C<error_input ()> can be called
-to retrieve the invalid argument.
-
-You may use the I<types ()> method for setting column types. See the
-description below.
-
 =head2 getline
 
  $colref = $csv->getline ($io);
@@ -1308,6 +1308,20 @@ reference to an empty list.
 The I<$csv-E<gt>string ()>, I<$csv-E<gt>fields ()> and I<$csv-E<gt>status ()>
 methods are meaningless, again.
 
+=head2 parse
+
+ $status = $csv->parse ($line);
+
+This object function decomposes a CSV string into fields, returning
+success or failure.  Failure can result from a lack of argument or the
+given CSV string is improperly formatted.  Upon success, C<fields ()> can
+be called to retrieve the decomposed fields .  Upon failure, the value
+returned by C<fields ()> is undefined and C<error_input ()> can be called
+to retrieve the invalid argument.
+
+You may use the I<types ()> method for setting column types. See the
+description below.
+
 =head2 getline_hr
 
 The C<getline_hr ()> and C<column_names ()> methods work together to allow
@@ -1338,19 +1352,19 @@ string C<"\cAUNDEF\cA">, so
   $hr = $csv->getline_hr ($io);
 
 Will set C<$hr->{"\cAUNDEF\cA"}> to the 1st field, C<$hr->{""}> to the
-2nd field, and C<$hr->{name}> to the 4th field, discarding the 2rd field.
+2nd field, and C<$hr->{name}> to the 4th field, discarding the 3rd field.
 
 C<column_names ()> croaks on invalid arguments.
 
 =head2 bind_columns
 
-Takes a list of references to scalars (max 255) to store the fields fetched
+Takes a list of references to scalars to store the fields fetched
 C<getline ()> in. When you don't pass enough references to store the
 fetched fields in, C<getline ()> will fail. If you pass more than there are
 fields to return, the remaining references are left untouched.
 
   $csv->bind_columns (\$code, \$name, \$price, \$description);
-  while ($csv->getline ()) {
+  while ($csv->getline ($io)) {
       print "The price of a $name is \x{20ac} $price\n";
       }
 
@@ -1408,7 +1422,11 @@ Set field type to string.
  @columns = $csv->fields ();
 
 This object function returns the input to C<combine ()> or the resultant
-decomposed fields of C<parse ()>, whichever was called more recently.
+decomposed fields of C successful <parse ()>, whichever was called more
+recently.
+
+Note that the return value is undefined after using C<getline ()>, which
+does not fill the data structures returned by C<parse ()>.
 
 =head2 meta_info
 
@@ -1474,6 +1492,7 @@ C<combine ()> or C<parse ()>, whichever was called more recently.
 
 =head2 error_diag
 
+ Text::CSV_PP->error_diag ();
  $csv->error_diag ();
  $error_code   = 0  + $csv->error_diag ();
  $error_str    = "" . $csv->error_diag ();
